/**
 * apeman react package for image component.
 * @class ApImage
 */

'use strict'

import React, {PropTypes as types} from 'react'
import ReactDOM from 'react-dom'
import classnames from 'classnames'
import numcal from 'numcal'
import scaledSize from './sizing/scaled_size'
import {ApSpinner} from 'apeman-react-spinner'
import {ApPureMixin} from 'apeman-react-mixins'

/** @lends ApImage */
const ApImage = React.createClass({

  // --------------------
  // Specs
  // --------------------

  propTypes: {
    /** Image scaling policy */
    scale: types.oneOf([
      'fit',
      'fill',
      'none'
    ]),
    /** Image width */
    width: types.oneOfType([ types.number, types.string ]),
    /** Image height */
    height: types.oneOfType([ types.number, types.string ]),
    /** Image src string */
    src: types.string,
    /** Alt test */
    alt: types.string,
    /** Them of spinner */
    spinnerTheme: types.string,
    /** Handler on image load */
    onLoad: types.func,
    /** Handler on image error. */
    onError: types.func
  },

  mixins: [
    ApPureMixin
  ],

  statics: {
    scaledSize,
    zeroIfNaN (value) {
      return isNaN(value) ? 0 : value
    },
    nullIfNaN (value) {
      return isNaN(value) ? null : value
    }
  },

  getInitialState () {
    const s = this
    return {
      imgWidth: null,
      imgHeight: null,
      mounted: false,
      ready: false,
      loading: !!s.props.src,
      error: null
    }
  },

  getDefaultProps () {
    return {
      scale: 'none',
      width: null,
      height: null,
      src: null,
      alt: 'NO IMAGE',
      spinnerTheme: ApSpinner.DEFAULT_THEME,
      onLoad: null,
      onError: null
    }
  },

  render () {
    const s = this
    let { state, props } = s

    let size = {
      width: props.width || null,
      height: props.height || null
    }

    let { mounted, error, ready, loading } = state
    return (
      <div className={ classnames('ap-image', props.className, {
                'ap-image-loading': props.src && loading,
                'ap-image-ready': props.src && ready
            }) }
           style={ Object.assign({}, size, props.style) }>
        { mounted && error ? s._renderNotfound(size) : null}
        { mounted && !error ? s._renderImg(size, mounted) : null }
        { loading ? s._renderSpinner(size) : null }
      </div>
    )
  },

  // --------------------
  // Lifecycle
  // --------------------

  componentWillMount () {
    const s = this
  },

  componentDidMount () {
    const s = this
    s.setState({
      mounted: true
    })

    setTimeout(() => s.resizeImage(), 0)
  },

  componentWillReceiveProps (nextProps) {
    const s = this
    let { props }  = s
    let srcChanged = nextProps.src !== props.src
    if (srcChanged) {
      s.setState({
        ready: false,
        loading: true,
        error: null
      })
      setTimeout(() => s.resizeImage(), 0)
      return
    }
    let widthChanged = nextProps.width !== props.width
    let heightChanged = nextProps.height !== props.height
    if (widthChanged || heightChanged) {
      setTimeout(() => s.resizeImage(), 0)
    }
  },

  // ------------------
  // Helper
  // ------------------

  handleLoad (e) {
    const s = this
    let { props } = s

    if (props.onLoad) {
      props.onLoad(e)
    }

    s.resizeImage(e.target.width, e.target.height)
  },

  handleError (e) {
    const s = this
    let { props } = s

    s.setState({
      error: e,
      loading: false
    })

    if (props.onError) {
      props.onError(e)
    }
  },

  resizeImage (imgContentWidth, imgContentHeight) {
    const s = this
    let { state, props } = s

    imgContentWidth = imgContentWidth || state.imgContentWidth
    imgContentHeight = imgContentHeight || state.imgContentHeight

    let valid = imgContentWidth && imgContentHeight
    if (!valid) {
      return
    }

    let elm = ReactDOM.findDOMNode(s)
    let frameSize = {
      width: elm.offsetWidth,
      height: elm.offsetHeight
    }
    let contentSize = {
      height: imgContentHeight,
      width: imgContentWidth
    }
    let scaledSize = ApImage.scaledSize(
      contentSize, frameSize, props.scale
    )

    s.setState({
      imgContentWidth,
      imgContentHeight,
      imgWidth: scaledSize.width,
      imgHeight: scaledSize.height,
      ready: true,
      loading: false
    })
  },

  // ------------------
  // Private
  // ------------------
  _renderImg (size) {
    const s = this
    let { state, props } = s

    let { nullIfNaN, zeroIfNaN } = ApImage

    return (
      <img src={ props.src }
           alt={ props.alt }
           className={ classnames('ap-image-content') }
           style={ {
                        top: zeroIfNaN((size.height - state.imgHeight) / 2),
                        left: zeroIfNaN((size.width - state.imgWidth) / 2),
                        width: nullIfNaN(state.imgWidth),
                        height: nullIfNaN(state.imgHeight)
                     } }
           onLoad={ s.handleLoad }
           onError={ s.handleError }
      />
    )
  },

  _renderNotfound (size) {
    const s = this
    let { props } = s

    return (
      <div className="ap-image-notfound"
           style={ {
                    lineHeight: `${size.height}px`,
                    fontSize: `${numcal.min(size.height * 0.4, 18)}px`
                 } }
      >{ props.alt }</div>
    )
  },

  _renderSpinner (size) {
    const s = this
    let { props } = s

    return (
      <ApSpinner className="ap-image-spinner"
                 theme={ props.spinnerTheme }
                 style={ {
                       width: size.width,
                       height: size.height
                       } }/>
    )
  }
})

export default ApImage
